<?php

/**
 * @file
 * Optional automatic settings configuration.
 *
 * @author valthebald <http://drupal.org/user/239562>
 */

use Drupal\Core\Site\Settings;
use Drupal\taxonomy\Entity\Vocabulary;

/**
 * Admin page for inputting auto-settings for meta tags.
 *
 * @param $form
 * @param $form_state
 */
function metatags_quick_admin_auto_settings($form, $form_state) {
  $current_settings = Settings::get('metatags_quick_auto_settings', '');

  // Construct the options array for all content types.
  $type_names = node_type_get_names();

  // Facility to select content types for auto-tagging. Has AJAX behavior.
  $form['applicable_content_types'] = [
    '#type' => 'checkboxes',
    '#title' => t('Applicable content types'),
    '#options' => $type_names,
    '#default_value' => $current_settings['applicable_content_types'] ?? [],
    '#description' => t('Select the content types for which auto settings should be applied'),
    '#ajax' => [
      'callback' => 'metatags_quick_extra_ajax_callback',
      'wrapper' => 'metatags-quick-auto-default-keywords',
      'method' => 'replace',
    ],
  ];

  // Common keywords that would be applied for all chosen content types.
  $form['global_keywords'] = [
    '#type' => 'textfield',
    '#title' => t('Global keywords'),
    '#maxlength' => Settings::get('metatags_quick_default_field_length', 255),
    '#default_value' => $current_settings['global_keywords'] ?? '',
    '#description' => t('Enter a comma separated list of keywords. Global keywords will be always added to applicable content types.'),
  ];

  // Content-type-wise special keywords. Only those content types chosen for auto-tagging would be displayed.
  $form['default_keywords'] = [
    '#type' => 'fieldset',
    '#title' => t('Default keywords'),
    '#collapsible' => TRUE,
    '#description' => t('Enter a comma separated list of keywords for the content types below, if you want them to be included automatifcally by default.'),
    '#tree' => TRUE,
    '#prefix' => '<div id="metatags-quick-auto-default-keywords">',
    '#suffix' => '</div>',
  ];

  $applicable_content_types = [];
  $available_content_types = [];

  if (!empty($form_state['values']['applicable_content_types'])) {
    $applicable_content_types = array_values($form_state['values']['applicable_content_types']);
  }
  elseif (isset($current_settings['applicable_content_types'])) {
    $applicable_content_types = array_values($current_settings['applicable_content_types']);
  }

  foreach ($type_names as $machine_name => $type_name) {
    $exists = in_array($machine_name, $applicable_content_types, TRUE);
    if ($exists) {
      $available_content_types[$machine_name] = $type_name;
    }
  }

  foreach ($available_content_types as $machine_name => $type_name) {
    if (isset($current_settings['default_keywords']) && isset($current_settings['default_keywords'][$machine_name])) {
      $default_value = $current_settings['default_keywords'][$machine_name];
    }
    else {
      $default_value = '';
    }

    $form['default_keywords'][$machine_name] = [
      '#type' => 'textfield',
      '#title' => $type_name,
      '#maxlength' => Settings::get('metatags_quick_default_field_length', 255),
      '#default_value' => $default_value,
    ];
  }

  // Taxonomy vocabularies that would supply keywords.
  $vocabularies = Vocabulary::loadMultiple();
  $options = [];
  foreach ($vocabularies as $machine_name => $vocabulary) {
    $options[$machine_name] = $vocabulary->name;
  }

  $form['auto_keywords_vocabularies'] = [
    '#type' => 'checkboxes',
    '#title' => t('Auto-keywords vocabularies'),
    '#options' => $options,
    '#default_value' => $current_settings['auto_keywords_vocabularies'] ?? [],
    '#description' => t('Select the vocabularies which contain terms you want to add to the keywords meta tag for nodes. The terms of these vocabularies are added before the global and default keywords but after the page-specific keywords.'),
  ];

  // Auto settings for meta description.
  $form['meta_description'] = [
    '#type' => 'fieldset',
    '#title' => t('Meta Description'),
    '#collapsible' => TRUE,
    '#description' => t('These options specify how meta description is to be generated from the node content.'),
    '#tree' => TRUE,
  ];

  if (isset($current_settings['meta_description']) && isset($current_settings['meta_description']['auto_generate'])) {
    $default_value = $current_settings['meta_description']['auto_generate'];
  }
  else {
    $default_value = '';
  }

  // Is auto-generation needed?
  $form['meta_description']['auto_generate'] = [
    '#type' => 'checkbox',
    '#title' => t('Auto generate meta description'),
    '#default_value' => $default_value,
  ];

  // If yes, from where?
  if (isset($current_settings['meta_description']) && isset($current_settings['meta_description']['source'])) {
    $default_value = $current_settings['meta_description']['source'];
  }
  else {
    $default_value = '';
  }

  $options = [
    'body' => t('Generate meta tags content from the node body'),
    'teaser' => t('Generate meta tags content from the node teaser'),
    'either' => t('Generate meta tags content from the node teaser, or the node body when the node teaser is empty'),
  ];

  $form['meta_description']['source'] = [
    '#type' => 'radios',
    '#title' => t('Generation source'),
    '#options' => $options,
    '#default_value' => $default_value,
  ];

  $form['op'] = [
    '#value' => t('Submit'),
    '#type' => 'submit',
  ];
  return $form;
}

/***
 * Submit function for the admin settings form.
 */
function metatags_quick_admin_auto_settings_submit($form, $form_state) {
  $new_settings = [
    'applicable_content_types' => $form_state['values']['applicable_content_types'],
    'global_keywords' => $form_state['values']['global_keywords'],
    'auto_keywords_vocabularies' => $form_state['values']['auto_keywords_vocabularies'],
  ];

  foreach ($form_state['values']['default_keywords'] as $machine_name => $value) {
    $new_settings['default_keywords'][$machine_name] = $value;
  }

  foreach ($form_state['values']['meta_description'] as $key => $value) {
    $new_settings['meta_description'][$key] = $value;
  }

  \Drupal::state()->set('metatags_quick_auto_settings', $new_settings);
  \Drupal::messenger()->addStatus(t('Auto settings saved for Meta tags (quick)'));
}

/**
 * Auto Settings form - AJAX Callback to display/remove textfields for entering content-type-wise keywords.
 *
 * @param $form
 * @param $form_state
 */
function metatags_quick_ajax_callback($form, $form_state) {
  return $form['default_keywords'];
}
